// Description: a value node that computes the difference of its children
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> all children must be value nodes
//<li> child value nodes of any size can be used
//<li> see the <a href="value.html">value</a> node for more information
//</ul>
//
// Category: Math
// Author: Alex Hill
// Revision: 11/01/01
//
#include "subtract.h"

extern "C" ygNode* APIENTRY construct_subtract(const char* name,bool master) { return new subtract(name,master); }
char* subtract_dependencies = "YG DSO Dependencies: value";

subtract::subtract(const char* name,bool master) : value(name,master)
    {
    setClassName("subtract");
    }

void subtract::app(void)
    {
    //if this node has size greater than zero
    if (size > 0)
        {
        int currNumChildren = numChildren();
        //for each child node do
        for (int i=0;i<currNumChildren;i++)
            {
            if (child(i) && child(i)->isOfClass("value"))
                {
                value* currChild = (value*)child(i);
                //get the size of the child node
                int currSize = currChild->getSize();
                //for each value in the child node do
                for (int j=0;j<currSize;j++)
                    {
                    //if the element is less than the size of this node then
                    if (j < size)
                        {
                        //if the first child, initialize the element value
                        if (i == 0)
                            values[j] = currChild->getValue(j);
                        //else, subtract the child value from the element value
                        else
                            values[j] -= currChild->getValue(j);
                        }
                    }
                }
            }
        }
    value::app();
    }
