// Description: creates a tracked simple transform from a CAVE sensor
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> this node can also be created with the alias: CAVETracker
//<li> this node will usually be a child of a <a href="ygUser.html">ygUser</a> node
//</ul>
//
// Category: User
// Author: Dave Pape
//           11/01/01
//           02/01/06 Alex Hill - added sensor number and logic
// Revision: 07/01/06 Alex Hill - removed updating of ygUser sensor number
//
#include <pfcave.h>
#include "ygNetMessages.h"
#include "ygUser.h"
#include "ygCAVETracker.h"

#include "ygDLL.h"

extern "C" YG_DLL_API ygNode* APIENTRY construct_ygCAVETracker(const char* name,bool master) { return new ygCAVETracker(name,master); }

ygCAVETracker::ygCAVETracker(const char* name,bool master) : ygTransform(name,master)
    {
    setClassName("ygCAVETracker",true);
    sensor_ = 0;
    caveTracking = false;
    addNetMessage("sensor",&sensor_,YG_NET_INT);
    addNetMessage("master", &netMaster, YG_NET_STRING);
    //dont include in documentation
    dontSaveNetMessage("master");
    }


void ygCAVETracker::reset(void)
    {
    //set sensor number to 0
    resetNetMessage("sensor");
    if (CAVEConfig->Distribution && isNetMaster())
        {
        netMaster = CAVEConfig->DistribTCPMaster;
        netMessageChanged("master");
        }
    ygTransform::reset();
    }


void ygCAVETracker::message(const ygMessage& msg)
    {
    //set the sensor number
    if (msg == "sensor")
        {
        return;
        }
    //set the host name of the net master node
    else if (msg == "master")
        {
        if (!CAVEDistribMaster())
            {
            if (netMaster == CAVEConfig->DistribTCPMaster)
                {
                dontReceiveNetMessage(COORD_NETKEY);
                caveTracking = true;
                }
            }
        }
    else
        ygTransform::message(msg);
    }


void ygCAVETracker::app(void)
    {
    if (isNetMaster() || caveTracking)
        {
        pfVec3 pos, ori;
        //get the CAVE sensor position
        CAVEGetSensorPosition(CAVESENSOR(sensor_), CAVE_TRACKER_FRAME, pos.vec);
        //get the CAVE sensor orientation
        CAVEGetSensorOrientation(CAVESENSOR(sensor_), CAVE_TRACKER_FRAME, ori.vec);
        //set the position and orientation of the transform
        setPosition(pos);
        setOrientation(ori);
        }
    ygTransform::app();
    }
