// Author: Dave Pape
//           11/01/04
// Rebision: 07/01/06 Alex Hill - removed ygNet::init() and added ygPFFrameDelay()
//
// This file consists of the major "control" functions - ones
// called from main to initialize & shut down yg, and some
// related functions.
//
#include <Performer/pf.h>
#include <Performer/pfdu.h>
//#include "ygFileServer.h"
#include "ygDSOLoader.h"
#include "ygNet.h"
#include "ygNodeDB.h"
#include "ygControl.h"

#ifndef YG_VR_LIBRARY
#define YG_VR_LIBRARY
#endif

char * ygVersion = "Ygdrasil Version 0.4.4 " YG_VR_LIBRARY
                   " - compiled " __DATE__;

using namespace std;

void APIENTRY ygPrePFInit(void)
    {
#ifdef __sgi
    extern void ygSGIIOStreamInit(void);
    ygSGIIOStreamInit();
#endif
    cout << ygVersion << endl;
    }


void APIENTRY ygPrePFConfig(void)
    {
//    ygFileServer::initialize();
    ygDSOLoader::init();
    if (getenv("YG_PATH"))
        pfFilePath(getenv("YG_PATH"));
    pfdInitConverter("pfb");
    if (getenv("YG_PF_DBASE_CONVERTERS"))
        {
        ygString extensions(getenv("YG_PF_DBASE_CONVERTERS"));
        ygString *token;
        int i=0;
        while (token = extensions.nextToken(" \t",&i))
            {
            pfdInitConverter(token->c_str());
            delete token;
            }
        }
    pfMultiprocess(PFMP_APPCULLDRAW);
    if (getenv("YG_PF_MULTIPROCESS"))
        ygPFMultiprocess(getenv("YG_PF_MULTIPROCESS"));
/* This option is needed for Alive on the Grid */
    if (getenv("YG_PF_INIT_SMOKES"))
        pfuInitSmokes();
    ygNodeDB::init();
    }


void APIENTRY ygHalt(void)
    {
    ygNet::halt();
    }


void APIENTRY ygPFMultiprocess(const ygString& mpmode)
    {
    ygString token;
    int pfmpmode = PFMP_APPCULLDRAW, index=0;
    while (mpmode.nextToken("| \t\n", &index, token))
        {
        if (token == "PFMP_FORK_ISECT")
            pfmpmode |= PFMP_FORK_ISECT;
        else if (token == "PFMP_FORK_CULL")
            pfmpmode |= PFMP_FORK_CULL;
        else if (token == "PFMP_FORK_DRAW")
            pfmpmode |= PFMP_FORK_DRAW;
        else if (token == "PFMP_FORK_LPOINT")
            pfmpmode |= PFMP_FORK_LPOINT;
        else if (token == "PFMP_FORK_DBASE")
            pfmpmode |= PFMP_FORK_DBASE;
        else if (token == "PFMP_FORK_COMPUTE")
            pfmpmode |= PFMP_FORK_COMPUTE;
        else if (token == "PFMP_CULLoDRAW")
            pfmpmode |= PFMP_CULLoDRAW;
        else if (token == "PFMP_CULL_DL_DRAW")
            pfmpmode |= PFMP_CULL_DL_DRAW;
        else
            cerr << "WARNING: ygPFMultiprocess: unrecognized token '"
                 << token << "'\n";
        }
    pfMultiprocess(pfmpmode);
    }


void APIENTRY ygDelayPFFrame(int frames)
    {
    int frameNum = 0;
    while (frameNum != frames)
            {
            pfFrame();
            pfSync();
            ygNet::process();
            frameNum++;
            }
    }


static bool ygServerMode_ = false;

bool APIENTRY ygServerMode(void)
    {
    return ygServerMode_;
    }


void APIENTRY ygSetServerMode(bool val)
    {
    ygServerMode_ = val;
    }
