// Description: creates a navigator with navigation data for the user
//
//<b>notes:</b>
//<ul>
//<li> derived nodes inherit all messages from their base classes
//<li> see reset method for default settings
//<li> sets the navigator of the user above it for use in navigation
//<li> this node serves no purpose other than as a base class for <a href="ygCAVENavigator.html">ygCAVENavigator</a>
//</ul>
//
// Category: User
// Author: Dave Pape
//           11/01/01
//           06/28/05 Dave Pape - removed navData object
// Revision: 02/01/06 Alex Hill - added logic to cleanly remove ygNavigator objects from scene 
//
#include "ygUser.h"
#include "ygWorld.h"
#include "ygViewer.h"
#include "ygNavigator.h"


ygNavigator::ygNavigator(const char* name,bool master) : ygNode(name,master)
    {
    setClassName("ygNavigator",false);
    user_ = NULL;
    }

ygNavigator::~ygNavigator(void)
    {
    if (user_)
        user_->setNavigator(NULL);
    if (ygWorld::world()->viewer())
        ((ygViewer*)ygWorld::world()->viewer())->setNavigator(NULL);
    }


void ygNavigator::reset(void)
    {
    //find the user ancestor of this node
    user_ = ygUser::findUserAncestor(this);
    //if found set navigator of that user to this node
    if (user_)
        user_->setNavigator(this);
    if (ygWorld::world()->viewer())
        ((ygViewer*)ygWorld::world()->viewer())->setNavigator(this);
    position_.set(0,0,0);
    orientation_.set(0,0,0);
    ygNode::reset();
    }


void ygNavigator::app(void)
    {
    //update navigation data
    user_->setPosition(position_);
    user_->setOrientation(orientation_);
    ygNode::app();
    }
