#ifndef _ygUtil_h_
#define _ygUtil_h_

#include <Performer/pr/pfLinMath.h>
#include <iostream>
#include "ygString.h"

inline std::ostream& operator<< (std::ostream& s, const pfVec2& v)
        { return s << "(" << v[0] << ", " << v[1] << ")"; }
inline std::ostream& operator<< (std::ostream& s, const pfVec3& v)
        { return s << "(" << v[0] << ", " << v[1] << ", " << v[2] << ")"; }
inline std::ostream& operator<< (std::ostream& s, const pfVec4& v)
        { return s << "(" << v[0] << ", " << v[1] << ", " << v[2]
                 << ", " << v[3] << ")"; }
inline std::ostream& operator<< (std::ostream& s, const pfMatrix& m)
        { return s << "[ " << m[0][0] << " " << m[0][1] << " " << m[0][2] << " " << m[0][3] << "]\n"
            << "[ " << m[1][0] << " " << m[1][1] << " " << m[1][2] << " " << m[1][3] << "]\n"
            << "[ " << m[2][0] << " " << m[2][1] << " " << m[2][2] << " " << m[2][3] << "]\n"
            << "[ " << m[3][0] << " " << m[3][1] << " " << m[3][2] << " " << m[3][3] << "]\n"; }


#include "ygDLL.h"

#ifdef WIN32

#ifdef __cplusplus
extern "C"
{
#endif // __cplusplus

#endif

extern YG_DLL_API bool APIENTRY ygFindFileInYgPath(const ygString& filename, ygString& returnPath);
extern YG_DLL_API bool APIENTRY ygFindFile(const ygString& filename, const ygString& searchPath,
        ygString& returnPath);
extern YG_DLL_API float APIENTRY ygGetTime(void);
extern YG_DLL_API double APIENTRY ygGetTimeD(void);

extern YG_DLL_API class pfGeoSet * APIENTRY ygCreateVolumeGeoset(class ygVolume * vol, int style, pfVec4* color=NULL);
extern YG_DLL_API class  pfGeoSet* APIENTRY ygCreateBoxGeoset(class ygBox* box,int style,pfVec4* color=NULL);
extern YG_DLL_API class  pfGeoSet* APIENTRY ygCreateSphereGeoset(class ygSphere* sphere,int style,pfVec4* color=NULL);
extern YG_DLL_API class  pfGeoSet* APIENTRY ygCreateCylinderGeoset(class ygCylinder* cyl,int style,pfVec4* color=NULL);

enum {
    YG_STYLE_WIREFRAME,
    YG_STYLE_SOLID
    };

extern YG_DLL_API void APIENTRY ygPrintSceneGraph(class ygNode* node,int max=99);

typedef YG_DLL_API void * (*ygThreadFuncPtr) (void *);
extern YG_DLL_API bool APIENTRY ygCreateThread(ygThreadFuncPtr,void *);

extern YG_DLL_API void APIENTRY ygMakeValidNodeName(ygString& name);
extern YG_DLL_API void APIENTRY ygIncrementName(ygString& name);

#ifdef WIN32
extern YG_DLL_API void APIENTRY ygInit();

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // WIN32


#endif
