// Author: Dave Pape
// Revision: 11/01/01
//
#include <Performer/pf/pfGroup.h>
#include <Performer/pr/pfGeoSet.h>
#include <Performer/pfdu.h>
#include "ygVolume.h"
#include "ygBox.h"
#include "ygCylinder.h"
#include "ygSphere.h"
#include "ygPointVolume.h"
#include "ygInfiniteVolume.h"
#include "ygUtil.h"

/****** GeoSet creation *****/

pfGeoSet * APIENTRY ygCreateVolumeGeoset(ygVolume * vol, int style, pfVec4* color)
    {
    if (vol->volumeType() == YGVOL_BOX)
        return ygCreateBoxGeoset((ygBox*)vol,style,color);
    else if (vol->volumeType() == YGVOL_SPHERE)
        return ygCreateSphereGeoset((ygSphere*)vol,style,color);
    else if (vol->volumeType() == YGVOL_CYLINDER)
        return ygCreateCylinderGeoset((ygCylinder*)vol,style,color);
    return NULL;
    }


pfGeoSet* APIENTRY ygCreateBoxGeoset(ygBox* box, int style, pfVec4* color)
    {
    pfGeoSet *gset = new pfGeoSet;
    int *lengths = (int *) pfMalloc(6 * sizeof(int),pfGetSharedArena());
    gset->setPrimType(PFGS_POLYS);
    gset->setNumPrims(6);
    lengths[0] = 4;
    lengths[1] = 4;
    lengths[2] = 4;
    lengths[3] = 4;
    lengths[4] = 4;
    lengths[5] = 4;
    gset->setPrimLengths(lengths);
    pfVec3 * verts = (pfVec3 *) pfMalloc(8 * sizeof(pfVec3),pfGetSharedArena());
    verts[0] = box->min;
    verts[1][0] = box->max[0]; verts[1][1] = box->min[1]; verts[1][2] = box->min[2];
    verts[2][0] = box->max[0]; verts[2][1] = box->max[1]; verts[2][2] = box->min[2];
    verts[3][0] = box->min[0]; verts[3][1] = box->max[1]; verts[3][2] = box->min[2];
    verts[4][0] = box->min[0]; verts[4][1] = box->min[1]; verts[4][2] = box->max[2];
    verts[5][0] = box->max[0]; verts[5][1] = box->min[1]; verts[5][2] = box->max[2];
    verts[6] = box->max;
    verts[7][0] = box->min[0]; verts[7][1] = box->max[1]; verts[7][2] = box->max[2];
    ushort *ilist_v = (ushort *) pfMalloc(24 * sizeof(ushort),pfGetSharedArena());
    ilist_v[0] = 0; ilist_v[1] = 3; ilist_v[2] = 2; ilist_v[3] = 1;
    ilist_v[4] = 0; ilist_v[5] = 1; ilist_v[6] = 5; ilist_v[7] = 4; 
    ilist_v[8] = 1; ilist_v[9] = 2; ilist_v[10] = 6; ilist_v[11] = 5;
    ilist_v[12] = 2; ilist_v[13] = 3; ilist_v[14] = 7; ilist_v[15] = 6;
    ilist_v[16] = 3; ilist_v[17] = 0; ilist_v[18] = 4; ilist_v[19] = 7;
    ilist_v[20] = 4; ilist_v[21] = 7; ilist_v[22] = 6; ilist_v[23] = 5;
    gset->setAttr(PFGS_COORD3, PFGS_PER_VERTEX, verts, ilist_v);
    if (!color)
        {
        color = (pfVec4 *) pfMalloc(sizeof(pfVec4),pfGetSharedArena());
        color->set(1,1,1,1);
        }    
    ushort *ilist_c = (ushort *) pfMalloc(1 * sizeof(ushort),pfGetSharedArena());
    ilist_c[0] = 0;
    gset->setAttr(PFGS_COLOR4, PFGS_OVERALL, color, ilist_c);
    gset->setGState(new pfGeoState);
    if (style == YG_STYLE_WIREFRAME)
        {
        gset->setLineWidth(1.0);
        gset->setDrawMode(PFGS_WIREFRAME,1);
        gset->getGState()->setMode(PFSTATE_ENWIREFRAME,1);
        }
    return gset;
    }


pfGeoSet* APIENTRY ygCreateSphereGeoset(ygSphere* sphere, int style, pfVec4* color)
    {
    pfGeoSet *gset;
    pfMatrix matrix;
    gset = pfdNewSphere(32,pfGetSharedArena());
    if (!color)
        {
        color = (pfVec4 *) pfMalloc(sizeof(pfVec4),pfGetSharedArena());
        color->set(1,1,1,1);
        }    
    gset->setAttr(PFGS_COLOR4, PFGS_OVERALL, color, NULL);
    gset->setGState(new pfGeoState);
    if (style == YG_STYLE_WIREFRAME)
        {
        gset->setLineWidth(1.0);
        gset->setDrawMode(PFGS_WIREFRAME,1);
        gset->getGState()->setMode(PFSTATE_ENWIREFRAME,1);
        }
    matrix.makeScale(sphere->radius,sphere->radius,sphere->radius);
    matrix.postTrans(matrix,sphere->center[0],sphere->center[1],
                sphere->center[2]);
    pfdXformGSet(gset,matrix);
    return gset;
    }


pfGeoSet* APIENTRY ygCreateCylinderGeoset(ygCylinder * cyl, int style, pfVec4* color)
    {
    pfGeoSet *gset;
    pfVec3 upVec(0,0,1);
    pfMatrix matrix, rotMatrix;
    gset = pfdNewPipe(1.0, 1.0, 24,pfGetSharedArena());
    if (!color)
        {
        color = (pfVec4 *) pfMalloc(sizeof(pfVec4),pfGetSharedArena());
        color->set(1,1,1,1);
        }    
    gset->setAttr(PFGS_COLOR4, PFGS_OVERALL, color, NULL);
    gset->setGState(new pfGeoState);
    if (style == YG_STYLE_WIREFRAME)
        {
        gset->setLineWidth(1.0);
        gset->setDrawMode(PFGS_WIREFRAME,1);
        gset->getGState()->setMode(PFSTATE_ENWIREFRAME,1);
        }
    matrix.makeScale(cyl->radius,cyl->radius,cyl->halfLength);
    rotMatrix.makeVecRotVec(upVec, cyl->axis);
    matrix *= rotMatrix;
    matrix.postTrans(matrix, cyl->center[0], cyl->center[1],
                 cyl->center[2]);
    pfdXformGSet(gset,matrix);
    return gset;
    }
